﻿/*
 * Copyright (c) 2013 CodingBlocks.NET <http://codingblocks.net>, Michael Outlaw <michael@michaeloutlaw.com>
 * 
 * See the file license.txt for copying permission.
 */

using System;
using System.Collections.Generic;
using System.Linq;

namespace SomeLibrary
{
    public class SomeInterfaceFactory
    {
        private static volatile IList<ISomeInterface> _someInterfaceInstances;
        private static readonly Object _lockSomeInterfaceInstances = new Object();
        public static IList<ISomeInterface> SomeInterfaceInstances
        {
            get
            {
                if (_someInterfaceInstances == null)
                {
                    var someInterfaceInstances = SomeInterfaceTypes.Select(CreateInstance).ToList();

                    lock (_lockSomeInterfaceInstances)
                    {
                        if (_someInterfaceInstances == null)
                        {
                            _someInterfaceInstances = someInterfaceInstances;
                        }
                    }
                }
                return _someInterfaceInstances;
            }
        }

        private static volatile IList<Type> _someInterfaceTypes;
        private static readonly Object _lockSomeInterfaceTypes = new Object();
        private static IList<Type> SomeInterfaceTypes
        {
            get
            {
                if (_someInterfaceTypes == null)
                {
                    var someInterfaceTypes = new List<Type>();
                    
                    foreach (var types in AppDomain.CurrentDomain.GetAssemblies().Select(assembly => assembly.GetTypes()))
                    {
                        someInterfaceTypes.AddRange(
                                types.Where(type => type.GetInterface(typeof(ISomeInterface).ToString()) != null)
                                    .ToList()
                                    .OrderBy(type => type.FullName));
                    }

                    lock (_lockSomeInterfaceTypes)
                    {
                        if (_someInterfaceTypes == null)
                        {
                            _someInterfaceTypes = someInterfaceTypes;
                        }
                    }
                }
                return _someInterfaceTypes;
            }
        }

        private static ISomeInterface CreateInstance(Type t)
        {
            if (t == null)
            {
                throw new ArgumentNullException("t", "A type of ISomeInterface is required!");
            }
            return Activator.CreateInstance(t, true) as ISomeInterface;
        }
    }
}
